@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 //===-- wasmedge/ast/type.h - type class definitions ----------------------===//
 //
@@ -762,3 +762,28 @@ private:
 
 } // namespace AST
 } // namespace WasmEdge
+
+template <>
+struct fmt::formatter<WasmEdge::AST::FunctionType>
+    : fmt::formatter<std::string_view> {
+  fmt::format_context::iterator
+  format(const WasmEdge::AST::FunctionType &Type,
+         fmt::format_context &Ctx) const noexcept {
+    using namespace std::literals;
+
+    fmt::memory_buffer Buffer;
+
+    fmt::format_to(std::back_inserter(Buffer), "[ "sv);
+    for (auto &P : Type.getParamTypes()) {
+      fmt::format_to(std::back_inserter(Buffer), "{} "sv, P);
+    }
+    fmt::format_to(std::back_inserter(Buffer), "] -> [ "sv);
+    for (auto &R : Type.getReturnTypes()) {
+      fmt::format_to(std::back_inserter(Buffer), "{} "sv, R);
+    }
+    fmt::format_to(std::back_inserter(Buffer), "]"sv);
+
+    return formatter<std::string_view>::format(
+        std::string_view(Buffer.data(), Buffer.size()), Ctx);
+  }
+};
