@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 //===-- wasmedge/executor/executor.h - Executor class definition ----------===//
 //
@@ -152,6 +152,14 @@ public:
   Expect<std::unique_ptr<Runtime::Instance::ModuleInstance>>
   instantiateModule(Runtime::StoreManager &StoreMgr, const AST::Module &Mod);
 
+  Expect<std::unique_ptr<Runtime::Instance::ComponentInstance>>
+  instantiateComponent(Runtime::StoreManager &StoreMgr,
+                       const AST::Component::Component &Comp);
+  Expect<std::unique_ptr<Runtime::Instance::ComponentInstance>>
+  instantiateComponent(Runtime::StoreManager &StoreMgr,
+                       const AST::Component::Component &Comp,
+                       std::string_view Name);
+
   /// Instantiate and register a WASM module into a named module instance.
   Expect<std::unique_ptr<Runtime::Instance::ModuleInstance>>
   registerModule(Runtime::StoreManager &StoreMgr, const AST::Module &Mod,
@@ -160,6 +168,9 @@ public:
   /// Register an instantiated module into a named module instance.
   Expect<void> registerModule(Runtime::StoreManager &StoreMgr,
                               const Runtime::Instance::ModuleInstance &ModInst);
+  Expect<void>
+  registerComponent(Runtime::StoreManager &StoreMgr,
+                    const Runtime::Instance::ComponentInstance &CompInst);
 
   /// Register a host function which will be invoked before calling a
   /// host function.
@@ -260,6 +271,57 @@ private:
                            const AST::ExportSection &ExportSec);
   /// @}
 
+  /// @{
+  /// Instantiation of Component Instance.
+  Expect<std::unique_ptr<Runtime::Instance::ComponentInstance>>
+  instantiate(Runtime::StoreManager &StoreMgr,
+              const AST::Component::Component &Comp,
+              std::optional<std::string_view> Name = std::nullopt);
+
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::CoreInstanceSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::CoreTypeSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::InstanceSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::AliasSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::TypeSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::CanonSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::StartSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::ImportSection &);
+  Expect<void> instantiate(Runtime::StoreManager &StoreMgr,
+                           Runtime::Instance::ComponentInstance &CompInst,
+                           const AST::Component::ExportSection &);
+  /// @}
+
+  /// \name Helper Functions for canonical ABI
+  /// @{
+  std::unique_ptr<Runtime::Instance::FunctionInstance>
+  lifting(Runtime::Instance::ComponentInstance &Comp,
+          const WasmEdge::AST::Component::FuncType &FuncType,
+          Runtime::Instance::FunctionInstance *F,
+          Runtime::Instance::MemoryInstance *Memory,
+          Runtime::Instance::FunctionInstance *Realloc);
+
+  std::unique_ptr<Runtime::Instance::FunctionInstance>
+  lowering(Runtime::Instance::FunctionInstance *F,
+           Runtime::Instance::MemoryInstance *Memory,
+           Runtime::Instance::FunctionInstance *Realloc);
+  /// @}
+
   /// \name Helper Functions for block controls.
   /// @{
   /// Helper function for calling functions. Return the continuation iterator.
@@ -674,6 +736,17 @@ private:
   template <typename T> Expect<void> runVectorFloorOp(ValVariant &Val) const;
   template <typename T> Expect<void> runVectorTruncOp(ValVariant &Val) const;
   template <typename T> Expect<void> runVectorNearestOp(ValVariant &Val) const;
+
+  /// ======= Relaxed SIMD instructions =======
+  template <typename T>
+  Expect<void> runVectorRelaxedLaneselectOp(ValVariant &Val1,
+                                            const ValVariant &Val2,
+                                            const ValVariant &Mask) const;
+  inline Expect<void>
+  runVectorRelaxedIntegerDotProductOp(ValVariant &Val1,
+                                      const ValVariant &Val2) const;
+  inline Expect<void> runVectorRelaxedIntegerDotProductOpAdd(
+      ValVariant &Val1, const ValVariant &Val2, const ValVariant &C) const;
   /// ======= Atomic instructions =======
   Expect<void> runAtomicNotifyOp(Runtime::StackManager &StackMgr,
                                  Runtime::Instance::MemoryInstance &MemInst,
