@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 //===-- wasmedge/runtime/instance/memory.h - Memory Instance definition ---===//
 //
@@ -46,14 +46,14 @@ public:
                  uint32_t PageLim = UINT32_C(65536)) noexcept
       : MemType(MType), PageLimit(PageLim) {
     if (MemType.getLimit().getMin() > PageLimit) {
-      spdlog::error(
-          "Create memory instance failed -- exceeded limit page size: {}",
-          PageLimit);
-      return;
+      spdlog::error("Memory Instance: Limited {} page in configuration.",
+                    PageLimit);
+      MemType.getLimit().setMin(PageLimit);
     }
     DataPtr = Allocator::allocate(MemType.getLimit().getMin());
     if (DataPtr == nullptr) {
-      spdlog::error("Unable to find usable memory address");
+      spdlog::error("Memory Instance: Unable to find usable memory address.");
+      MemType.getLimit().setMin(0U);
       return;
     }
   }
@@ -105,7 +105,8 @@ public:
     }
     assuming(PageLimit >= Min);
     if (Count > PageLimit - Min) {
-      spdlog::error("Memory grow page failed -- exceeded limit page size: {}",
+      spdlog::error("Memory Instance: Memory grow page failed, exceeded "
+                    "limited {} page size in configuration.",
                     PageLimit);
       return false;
     }
