@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 //===-- wasmedge/runtime/instance/module.h - Module Instance definition ---===//
 //
@@ -13,6 +13,8 @@
 //===----------------------------------------------------------------------===//
 #pragma once
 
+#include "ast/component/type.h"
+#include "ast/module.h"
 #include "ast/type.h"
 #include "common/errcode.h"
 #include "runtime/hostfunc.h"
@@ -67,6 +69,8 @@ inline constexpr const bool IsInstanceV =
     std::is_same_v<T, Instance::DataInstance>;
 } // namespace
 
+class ComponentInstance;
+
 class ModuleInstance {
 public:
   ModuleInstance(std::string_view Name, void *Data = nullptr,
@@ -110,6 +114,7 @@ public:
     unsafeAddHostInstance(Name, OwnedFuncInsts, FuncInsts, ExpFuncs,
                           std::move(Func));
   }
+
   void addHostTable(std::string_view Name,
                     std::unique_ptr<TableInstance> &&Tab) {
     std::unique_lock Lock(Mutex);
@@ -200,8 +205,51 @@ public:
     return std::forward<CallbackT>(CallBack)(ExpGlobals);
   }
 
+  /// Component model concepts
+  ///
+  /// Export functions with name, these functions are suppose not owned by this
+  /// module, because the module is just a wrapper for component functions.
+  ///
+  /// See the example below, with statement below shows why we need this kind of
+  /// exporting
+  ///
+  /// (component
+  ///   (core module $A
+  ///     (func (export "one") (result i32) (i32.const 1))
+  ///   )
+  ///   (core module $B
+  ///     (func (import "a" "one") (result i32))
+  ///   )
+  ///   (core instance $a (instantiate $A))
+  ///   (core instance $b (instantiate $B (with "a" (instance $a))))
+  /// )
+  void exportFunction(std::string_view Name, FunctionInstance *Func) {
+    std::unique_lock Lock(Mutex);
+    assuming(Func->isHostFunction());
+    unsafeImportDefinedType(Func->getHostFunc().getDefinedType());
+    Func->linkDefinedType(this, static_cast<uint32_t>(Types.size()) - 1);
+    FuncInsts.push_back(Func);
+    ExpFuncs.insert_or_assign(std::string(Name), FuncInsts.back());
+  }
+  void exportTable(std::string_view Name, TableInstance *Tab) {
+    std::unique_lock Lock(Mutex);
+    TabInsts.push_back(Tab);
+    ExpTables.insert_or_assign(std::string(Name), TabInsts.back());
+  }
+  void exportMemory(std::string_view Name, MemoryInstance *Mem) {
+    std::unique_lock Lock(Mutex);
+    MemInsts.push_back(Mem);
+    ExpMems.insert_or_assign(std::string(Name), MemInsts.back());
+  }
+  void exportGlobal(std::string_view Name, GlobalInstance *Glob) {
+    std::unique_lock Lock(Mutex);
+    GlobInsts.push_back(Glob);
+    ExpGlobals.insert_or_assign(std::string(Name), GlobInsts.back());
+  }
+
 protected:
   friend class Executor::Executor;
+  friend class ComponentInstance;
   friend class Runtime::CallingFrame;
 
   /// Create and copy the defined type to this module instance.
