@@ -1,10 +1,13 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 #include "common/errinfo.h"
 #include "common/errcode.h"
 #include "common/hexstr.h"
 
+#include <spdlog/fmt/fmt.h>
+#include <spdlog/fmt/ranges.h>
+
 using namespace std::literals;
 
 fmt::format_context::iterator
