@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 //===-- wasmedge/test/api/hostfunc_c.c - Spec test host functions for C API ==//
 //
@@ -19,23 +19,22 @@
 
 #if defined(_MSC_VER) && !defined(__clang__) // MSVC
 #define MAYBE_UNUSED
-#pragma warning( disable : 4100 ) // unreferenced formal parameter
+#pragma warning(disable : 4100) // unreferenced formal parameter
 #else
 #define MAYBE_UNUSED __attribute__((unused))
 #endif
 
-WasmEdge_Result SpecTestPrint(void *Data MAYBE_UNUSED,
-                              const WasmEdge_CallingFrameContext *CallFrameCxt
-                              MAYBE_UNUSED,
-                              const WasmEdge_Value *In MAYBE_UNUSED,
-                              WasmEdge_Value *Out MAYBE_UNUSED) {
+WasmEdge_Result
+SpecTestPrint(void *Data MAYBE_UNUSED,
+              const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
+              const WasmEdge_Value *In MAYBE_UNUSED,
+              WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
 }
 
 WasmEdge_Result
 SpecTestPrintI32(void *Data MAYBE_UNUSED,
-                 const WasmEdge_CallingFrameContext *CallFrameCxt
-                 MAYBE_UNUSED,
+                 const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
                  const WasmEdge_Value *In MAYBE_UNUSED,
                  WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
@@ -43,8 +42,7 @@ SpecTestPrintI32(void *Data MAYBE_UNUSED,
 
 WasmEdge_Result
 SpecTestPrintI64(void *Data MAYBE_UNUSED,
-                 const WasmEdge_CallingFrameContext *CallFrameCxt
-                 MAYBE_UNUSED,
+                 const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
                  const WasmEdge_Value *In MAYBE_UNUSED,
                  WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
@@ -52,8 +50,7 @@ SpecTestPrintI64(void *Data MAYBE_UNUSED,
 
 WasmEdge_Result
 SpecTestPrintF32(void *Data MAYBE_UNUSED,
-                 const WasmEdge_CallingFrameContext *CallFrameCxt
-                 MAYBE_UNUSED,
+                 const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
                  const WasmEdge_Value *In MAYBE_UNUSED,
                  WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
@@ -61,28 +58,23 @@ SpecTestPrintF32(void *Data MAYBE_UNUSED,
 
 WasmEdge_Result
 SpecTestPrintF64(void *Data MAYBE_UNUSED,
-                 const WasmEdge_CallingFrameContext *CallFrameCxt
-                 MAYBE_UNUSED,
+                 const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
                  const WasmEdge_Value *In MAYBE_UNUSED,
                  WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
 }
 
-WasmEdge_Result
-SpecTestPrintI32F32(void *Data MAYBE_UNUSED,
-                    const WasmEdge_CallingFrameContext *CallFrameCxt
-                    MAYBE_UNUSED,
-                    const WasmEdge_Value *In MAYBE_UNUSED,
-                    WasmEdge_Value *Out MAYBE_UNUSED) {
+WasmEdge_Result SpecTestPrintI32F32(
+    void *Data MAYBE_UNUSED,
+    const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
+    const WasmEdge_Value *In MAYBE_UNUSED, WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
 }
 
-WasmEdge_Result
-SpecTestPrintF64F64(void *Data MAYBE_UNUSED,
-                    const WasmEdge_CallingFrameContext *CallFrameCxt
-                    MAYBE_UNUSED,
-                    const WasmEdge_Value *In MAYBE_UNUSED,
-                    WasmEdge_Value *Out MAYBE_UNUSED) {
+WasmEdge_Result SpecTestPrintF64F64(
+    void *Data MAYBE_UNUSED,
+    const WasmEdge_CallingFrameContext *CallFrameCxt MAYBE_UNUSED,
+    const WasmEdge_Value *In MAYBE_UNUSED, WasmEdge_Value *Out MAYBE_UNUSED) {
   return WasmEdge_Result_Success;
 }
 
@@ -100,7 +92,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_Limit TabLimit;
   WasmEdge_Limit MemLimit;
   WasmEdge_Limit SharedMemLimit;
-  enum WasmEdge_ValType Param[2];
+  WasmEdge_ValType Param[2];
 
   HostName = WasmEdge_StringCreateByCString("spectest");
   HostMod = WasmEdge_ModuleInstanceCreate(HostName);
@@ -115,7 +107,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host function "print_i32": {i32} -> {}
-  Param[0] = WasmEdge_ValType_I32;
+  Param[0] = WasmEdge_ValTypeGenI32();
   HostFType = WasmEdge_FunctionTypeCreate(Param, 1, NULL, 0);
   HostFunc =
       WasmEdge_FunctionInstanceCreate(HostFType, SpecTestPrintI32, NULL, 0);
@@ -125,7 +117,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host function "print_i64": {i64} -> {}
-  Param[0] = WasmEdge_ValType_I64;
+  Param[0] = WasmEdge_ValTypeGenI64();
   HostFType = WasmEdge_FunctionTypeCreate(Param, 1, NULL, 0);
   HostFunc =
       WasmEdge_FunctionInstanceCreate(HostFType, SpecTestPrintI64, NULL, 0);
@@ -135,7 +127,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host function "print_f32": {f32} -> {}
-  Param[0] = WasmEdge_ValType_F32;
+  Param[0] = WasmEdge_ValTypeGenF32();
   HostFType = WasmEdge_FunctionTypeCreate(Param, 1, NULL, 0);
   HostFunc =
       WasmEdge_FunctionInstanceCreate(HostFType, SpecTestPrintF32, NULL, 0);
@@ -145,7 +137,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host function "print_f64": {f64} -> {}
-  Param[0] = WasmEdge_ValType_F64;
+  Param[0] = WasmEdge_ValTypeGenF64();
   HostFType = WasmEdge_FunctionTypeCreate(Param, 1, NULL, 0);
   HostFunc =
       WasmEdge_FunctionInstanceCreate(HostFType, SpecTestPrintF64, NULL, 0);
@@ -155,8 +147,8 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host function "print_i32_f32": {i32, f32} -> {}
-  Param[0] = WasmEdge_ValType_I32;
-  Param[1] = WasmEdge_ValType_F32;
+  Param[0] = WasmEdge_ValTypeGenI32();
+  Param[1] = WasmEdge_ValTypeGenF32();
   HostFType = WasmEdge_FunctionTypeCreate(Param, 2, NULL, 0);
   HostFunc =
       WasmEdge_FunctionInstanceCreate(HostFType, SpecTestPrintI32F32, NULL, 0);
@@ -166,8 +158,8 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host function "print_f64_f64": {f64, f64} -> {}
-  Param[0] = WasmEdge_ValType_F64;
-  Param[1] = WasmEdge_ValType_F64;
+  Param[0] = WasmEdge_ValTypeGenF64();
+  Param[1] = WasmEdge_ValTypeGenF64();
   HostFType = WasmEdge_FunctionTypeCreate(Param, 2, NULL, 0);
   HostFunc =
       WasmEdge_FunctionInstanceCreate(HostFType, SpecTestPrintF64F64, NULL, 0);
@@ -181,7 +173,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   TabLimit.Shared = false;
   TabLimit.Min = 10;
   TabLimit.Max = 20;
-  HostTType = WasmEdge_TableTypeCreate(WasmEdge_RefType_FuncRef, TabLimit);
+  HostTType = WasmEdge_TableTypeCreate(WasmEdge_ValTypeGenFuncRef(), TabLimit);
   HostTable = WasmEdge_TableInstanceCreate(HostTType);
   WasmEdge_TableTypeDelete(HostTType);
   HostName = WasmEdge_StringCreateByCString("table");
@@ -213,7 +205,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host global "global_i32": const 666
-  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValType_I32,
+  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValTypeGenI32(),
                                         WasmEdge_Mutability_Const);
   HostGlobal =
       WasmEdge_GlobalInstanceCreate(HostGType, WasmEdge_ValueGenI32(666));
@@ -223,7 +215,7 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host global "global_i64": const 666
-  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValType_I64,
+  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValTypeGenI64(),
                                         WasmEdge_Mutability_Const);
   HostGlobal =
       WasmEdge_GlobalInstanceCreate(HostGType, WasmEdge_ValueGenI64(666));
@@ -233,20 +225,20 @@ WasmEdge_ModuleInstanceContext *createSpecTestModule(void) {
   WasmEdge_StringDelete(HostName);
 
   // Add host global "global_f32": const 666.0
-  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValType_F32,
+  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValTypeGenF32(),
                                         WasmEdge_Mutability_Const);
-  HostGlobal =
-      WasmEdge_GlobalInstanceCreate(HostGType, WasmEdge_ValueGenF32(666.0));
+  HostGlobal = WasmEdge_GlobalInstanceCreate(
+      HostGType, WasmEdge_ValueGenF32((float)666.6));
   WasmEdge_GlobalTypeDelete(HostGType);
   HostName = WasmEdge_StringCreateByCString("global_f32");
   WasmEdge_ModuleInstanceAddGlobal(HostMod, HostName, HostGlobal);
   WasmEdge_StringDelete(HostName);
 
   // Add host global "global_f64": const 666.0
-  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValType_F64,
+  HostGType = WasmEdge_GlobalTypeCreate(WasmEdge_ValTypeGenF64(),
                                         WasmEdge_Mutability_Const);
-  HostGlobal =
-      WasmEdge_GlobalInstanceCreate(HostGType, WasmEdge_ValueGenF64(666.0));
+  HostGlobal = WasmEdge_GlobalInstanceCreate(
+      HostGType, WasmEdge_ValueGenF64((double)666.6));
   WasmEdge_GlobalTypeDelete(HostGType);
   HostName = WasmEdge_StringCreateByCString("global_f64");
   WasmEdge_ModuleInstanceAddGlobal(HostMod, HostName, HostGlobal);
