@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
-// SPDX-FileCopyrightText: 2019-2022 Second State INC
+// SPDX-FileCopyrightText: 2019-2024 Second State INC
 
 #include "common/defines.h"
 #include "runtime/instance/module.h"
@@ -10,11 +10,23 @@
 #include <array>
 #include <cstdint>
 #include <gtest/gtest.h>
+#include <memory>
 #include <string>
 #include <vector>
 
 namespace {
-WasmEdge::Runtime::Instance::ModuleInstance *createModule() {
+
+template <typename T, typename U>
+inline std::unique_ptr<T> dynamicPointerCast(std::unique_ptr<U> &&R) noexcept {
+  static_assert(std::has_virtual_destructor_v<T>);
+  T *P = dynamic_cast<T *>(R.get());
+  if (P) {
+    R.release();
+  }
+  return std::unique_ptr<T>(P);
+}
+
+std::unique_ptr<WasmEdge::Host::WasmEdgeTensorflowModule> createModule() {
   using namespace std::literals::string_view_literals;
   WasmEdge::Plugin::Plugin::load(std::filesystem::u8path(
       "../../../plugins/wasmedge_tensorflow/" WASMEDGE_LIB_PREFIX
@@ -22,10 +34,11 @@ WasmEdge::Runtime::Instance::ModuleInstance *createModule() {
   if (const auto *Plugin =
           WasmEdge::Plugin::Plugin::find("wasmedge_tensorflow"sv)) {
     if (const auto *Module = Plugin->findModule("wasmedge_tensorflow"sv)) {
-      return Module->create().release();
+      return dynamicPointerCast<WasmEdge::Host::WasmEdgeTensorflowModule>(
+          Module->create());
     }
   }
-  return nullptr;
+  return {};
 }
 } // namespace
 
@@ -33,9 +46,9 @@ WasmEdge::Runtime::Instance::ModuleInstance *createModule() {
 
 TEST(WasmEdgeTensorflowTest, Module) {
   // Create the wasmedge_tensorflow module instance.
-  auto *TFMod =
-      dynamic_cast<WasmEdge::Host::WasmEdgeTensorflowModule *>(createModule());
-  EXPECT_FALSE(TFMod == nullptr);
+  auto TFMod = createModule();
+  ASSERT_TRUE(TFMod);
+
   EXPECT_EQ(TFMod->getFuncExportNum(), 11U);
   EXPECT_NE(TFMod->findFuncExports("create_session"), nullptr);
   EXPECT_NE(TFMod->findFuncExports("create_session_saved_model"), nullptr);
@@ -48,7 +61,6 @@ TEST(WasmEdgeTensorflowTest, Module) {
   EXPECT_NE(TFMod->findFuncExports("append_output"), nullptr);
   EXPECT_NE(TFMod->findFuncExports("clear_input"), nullptr);
   EXPECT_NE(TFMod->findFuncExports("clear_output"), nullptr);
-  delete TFMod;
 }
 
 GTEST_API_ int main(int argc, char **argv) {
