@@ -284,14 +284,14 @@ SUPPORTED_PLATFORM_MACHINE = {
 }
 
 SUPPORTED_MIN_VERSION = {
-    "Linux" + "x86_64": VersionString("0.10.0"),
-    "Linux" + "amd64": VersionString("0.10.0"),
-    "Linux" + "arm64": VersionString("0.10.0"),
-    "Linux" + "armv8": VersionString("0.10.0"),
-    "Linux" + "aarch64": VersionString("0.10.0"),
-    "Darwin" + "x86_64": VersionString("0.10.0"),
-    "Darwin" + "arm64": VersionString("0.10.0"),
-    "Darwin" + "arm": VersionString("0.10.0"),
+    "Linux" + "x86_64": VersionString("0.13.0"),
+    "Linux" + "amd64": VersionString("0.13.0"),
+    "Linux" + "arm64": VersionString("0.13.0"),
+    "Linux" + "armv8": VersionString("0.13.0"),
+    "Linux" + "aarch64": VersionString("0.13.0"),
+    "Darwin" + "x86_64": VersionString("0.13.0"),
+    "Darwin" + "arm64": VersionString("0.13.0"),
+    "Darwin" + "arm": VersionString("0.13.0"),
 }
 
 WASMEDGE = "WasmEdge"
@@ -301,37 +301,7 @@ TENSORFLOW_LITE = "tensorflow_lite"
 TENSORFLOW_LITE_P = "tensorflowlite"
 TENSORFLOW_DEPS = "tensorflow_deps"
 TENSORFLOW_LITE_DEPS = "tensorflow_lite_deps"
-TENSORFLOW_TOOLS = "tensorflow_tools"
 IMAGE = "image"
-EXTENSIONS = [TENSORFLOW, IMAGE]
-
-SUPPORTED_EXTENSIONS = {
-    "Linux" + "x86_64": EXTENSIONS,
-    "Linux" + "amd64": EXTENSIONS,
-    "Linux" + "arm64": EXTENSIONS,
-    "Linux" + "armv8": EXTENSIONS,
-    "Linux" + "aarch64": EXTENSIONS,
-    "Darwin" + "x86_64": EXTENSIONS,
-    "Darwin" + "arm64": EXTENSIONS,
-    "Darwin" + "arm": EXTENSIONS,
-}
-
-SUPPORTED_EXTENSIONS_VERSION = {
-    "Linux" + "x86_64" + TENSORFLOW: VersionString("0.10.0"),
-    "Linux" + "x86_64" + IMAGE: VersionString("0.10.0"),
-    "Linux" + "amd64" + TENSORFLOW: VersionString("0.10.0"),
-    "Linux" + "amd64" + IMAGE: VersionString("0.10.0"),
-    "Linux" + "arm64" + TENSORFLOW: VersionString("0.10.0"),
-    "Linux" + "arm64" + IMAGE: VersionString("0.10.0"),
-    "Linux" + "armv8" + TENSORFLOW: VersionString("0.10.0"),
-    "Linux" + "armv8" + IMAGE: VersionString("0.10.0"),
-    "Linux" + "aarch64" + TENSORFLOW: VersionString("0.10.0"),
-    "Linux" + "aarch64" + IMAGE: VersionString("0.10.0"),
-    "Darwin" + "x86_64" + TENSORFLOW: VersionString("0.10.0"),
-    "Darwin" + "x86_64" + IMAGE: VersionString("0.10.0"),
-    "Darwin" + "arm64" + TENSORFLOW: VersionString("0.10.0"),
-    "Darwin" + "arm" + TENSORFLOW: VersionString("0.10.0"),
-}
 
 WASI_NN_OPENVINO = "wasi_nn-openvino"
 WASI_CRYPTO = "wasi_crypto"
@@ -340,7 +310,7 @@ WASI_NN_TENSORFLOW_LITE = "wasi_nn-tensorflowlite"
 WASI_NN_GGML = "wasi_nn-ggml"
 WASI_NN_GGML_CUDA = "wasi_nn-ggml-cuda"
 WASI_NN_GGML_NOAVX = "wasi_nn-ggml-noavx"
-WASI_NN_GGML_BLAS = "wasi_nn-ggml-blas"
+WASI_LOGGING = "wasi_logging"
 WASMEDGE_TENSORFLOW_PLUGIN = WASMEDGE.lower() + "_" + TENSORFLOW
 WASMEDGE_TENSORFLOW_LITE_PLUGIN = WASMEDGE.lower() + "_" + TENSORFLOW_LITE_P
 WASMEDGE_IMAGE_PLUGIN = WASMEDGE.lower() + "_" + IMAGE
@@ -355,7 +325,7 @@ PLUGINS_AVAILABLE = [
     WASI_NN_GGML,
     WASI_NN_GGML_CUDA,
     WASI_NN_GGML_NOAVX,
-    WASI_NN_GGML_BLAS,
+    WASI_LOGGING,
     WASMEDGE_TENSORFLOW_PLUGIN,
     WASMEDGE_TENSORFLOW_LITE_PLUGIN,
     WASMEDGE_IMAGE_PLUGIN,
@@ -364,27 +334,25 @@ PLUGINS_AVAILABLE = [
 ]
 
 SUPPORTTED_PLUGINS = {
-    "ubuntu20.04" + "x86_64" + WASI_CRYPTO: VersionString("0.10.1-rc.1"),
-    "manylinux2014" + "x86_64" + WASI_CRYPTO: VersionString("0.10.1-rc.1"),
-    "manylinux2014" + "aarch64" + WASI_CRYPTO: VersionString("0.10.1-rc.1"),
-    "manylinux2014" + "arm64" + WASI_CRYPTO: VersionString("0.10.1-rc.1"),
-    "ubuntu20.04" + "x86_64" + WASI_NN_OPENVINO: VersionString("0.10.1-alpha.1"),
-    "ubuntu20.04" + "x86_64" + WASI_NN_PYTORCH: VersionString("0.11.1-alpha.1"),
+    "ubuntu20.04" + "x86_64" + WASI_CRYPTO: VersionString("0.13.0"),
+    "manylinux2014" + "x86_64" + WASI_CRYPTO: VersionString("0.13.0"),
+    "manylinux2014" + "aarch64" + WASI_CRYPTO: VersionString("0.13.0"),
+    "manylinux2014" + "arm64" + WASI_CRYPTO: VersionString("0.13.0"),
+    "ubuntu20.04" + "x86_64" + WASI_NN_OPENVINO: VersionString("0.13.0"),
+    "ubuntu20.04" + "x86_64" + WASI_NN_PYTORCH: VersionString("0.13.0"),
     "ubuntu20.04" + "x86_64" + WASI_NN_GGML: VersionString("0.13.4"),
     "ubuntu20.04" + "aarch64" + WASI_NN_GGML: VersionString("0.13.5"),
     "ubuntu20.04" + "x86_64" + WASI_NN_GGML_NOAVX: VersionString("0.13.5"),
     "ubuntu20.04" + "x86_64" + WASI_NN_GGML_CUDA: VersionString("0.13.4"),
     "ubuntu20.04" + "aarch64" + WASI_NN_GGML_CUDA: VersionString("0.13.5"),
-    "ubuntu20.04" + "x86_64" + WASI_NN_GGML_BLAS: VersionString("0.13.5"),
-    "ubuntu20.04" + "aarch64" + WASI_NN_GGML_BLAS: VersionString("0.13.5"),
-    "manylinux2014" + "x86_64" + WASI_NN_PYTORCH: VersionString("0.11.2-alpha.1"),
-    "manylinux2014" + "x86_64" + WASI_NN_TENSORFLOW_LITE: VersionString("0.10.0"),
+    "manylinux2014" + "x86_64" + WASI_NN_PYTORCH: VersionString("0.13.0"),
+    "manylinux2014" + "x86_64" + WASI_NN_TENSORFLOW_LITE: VersionString("0.13.0"),
     "manylinux2014" + "x86_64" + WASI_NN_GGML: VersionString("0.13.4"),
-    "manylinux2014" + "aarch64" + WASI_NN_TENSORFLOW_LITE: VersionString("0.10.0"),
+    "manylinux2014" + "aarch64" + WASI_NN_TENSORFLOW_LITE: VersionString("0.13.0"),
     "manylinux2014" + "aarch64" + WASI_NN_GGML: VersionString("0.13.4"),
     "darwin" + "x86_64" + WASI_NN_GGML: VersionString("0.13.4"),
     "darwin" + "arm64" + WASI_NN_GGML: VersionString("0.13.4"),
-    "ubuntu20.04" + "x86_64" + WASI_NN_TENSORFLOW_LITE: VersionString("0.11.2-rc.1"),
+    "ubuntu20.04" + "x86_64" + WASI_NN_TENSORFLOW_LITE: VersionString("0.13.0"),
     "darwin" + "x86_64" + WASMEDGE_TENSORFLOW_PLUGIN: VersionString("0.13.0"),
     "darwin" + "arm64" + WASMEDGE_TENSORFLOW_PLUGIN: VersionString("0.13.0"),
     "manylinux2014" + "x86_64" + WASMEDGE_TENSORFLOW_PLUGIN: VersionString("0.13.0"),
@@ -404,6 +372,12 @@ SUPPORTTED_PLUGINS = {
     "manylinux2014" + "x86_64" + WASMEDGE_IMAGE_PLUGIN: VersionString("0.13.0"),
     "manylinux2014" + "aarch64" + WASMEDGE_IMAGE_PLUGIN: VersionString("0.13.0"),
     "ubuntu20.04" + "x86_64" + WASMEDGE_IMAGE_PLUGIN: VersionString("0.13.0"),
+    "darwin" + "x86_64" + WASI_LOGGING: VersionString("0.14.0"),
+    "darwin" + "arm64" + WASI_LOGGING: VersionString("0.13.5"),
+    "manylinux2014" + "aarch64" + WASI_LOGGING: VersionString("0.13.5"),
+    "manylinux2014" + "x86_64" + WASI_LOGGING: VersionString("0.13.5"),
+    "ubuntu20.04" + "x86_64" + WASI_LOGGING: VersionString("0.13.5"),
+    "ubuntu20.04" + "aarch64" + WASI_LOGGING: VersionString("0.14.0"),
     "darwin" + "x86_64" + WASMEDGE_RUSTLS: VersionString("0.13.4"),
     "darwin" + "arm64" + WASMEDGE_RUSTLS: VersionString("0.13.4"),
     "manylinux2014" + "aarch64" + WASMEDGE_RUSTLS: VersionString("0.13.5"),
@@ -668,6 +642,19 @@ def check_nvidia_smi(platform):
     return False
 
 
+def check_libcudart(platform):
+    if platform == "Linux":
+        cmd = "ldconfig -p | grep libcudart.so"
+        output = run_shell_command(cmd)
+        logging.debug("%s: %s", cmd, output)
+        if "libcudart.so" in output:
+            return True
+        else:
+            logging.info("Cannot find libcudart.so")
+            return False
+    return False
+
+
 def ldconfig(args, compat):
     if geteuid() == 0:
         # Only run ldconfig or update_dyld_shared_cache when user is root/sudoer
@@ -690,142 +677,23 @@ def is_default_path(args):
     return args.path == abspath(PATH) or args.path[:4] != "/usr"
 
 
-def install_image_extension(args, compat):
-    global CONST_release_pkg, CONST_lib_dir
-
-    if not get_remote_version_availability(
-        "second-state/WasmEdge-image", args.image_version
-    ):
-        logging.error(
-            "Image extension version incorrect: {0}".format(args.image_version)
-        )
-        return -1
-    if compat.prefix() + IMAGE not in SUPPORTED_EXTENSIONS_VERSION:
-        logging.error("Image extensions not compatible: {0}".format(compat.prefix()))
-        return -1
-    elif (
-        SUPPORTED_EXTENSIONS_VERSION[compat.prefix() + IMAGE].compare(
-            args.image_version
-        )
-        > 0
-    ):
-        logging.error(
-            "Min image extensions version: {0}".format(
-                SUPPORTED_EXTENSIONS_VERSION[compat.prefix() + IMAGE],
-            )
-        )
-        return -1
-
-    logging.info("Downloading image extension")
-
-    local_release_package = CONST_release_pkg
-
-    # From WasmEdge 0.11.1, we have the Ubuntu release.
-    # Installation of ubuntu version extensions when the ubuntu version of WasmEdge selected.
-    if VersionString(args.image_version).compare("0.11.1") >= 0:
-        local_release_package = compat.release_package_wasmedge
-        logging.debug("Downloading dist package: {0}".format(local_release_package))
-
-    image_pkg = "WasmEdge-image-" + args.image_version + "-" + local_release_package
-
-    download_url(CONST_urls[IMAGE], join(TEMP_PATH, image_pkg), show_progress)
-
-    # Extract archive
-    extract_archive(
-        join(TEMP_PATH, image_pkg),
-        args.path,
-        join(TEMP_PATH, "WasmEdge-image"),
-        env_file_path=CONST_env_path,
-        remove_finished=True,
-    )
-
-    wasmedge_image_temp = join(TEMP_PATH, "WasmEdge-image")
-    for dir in listdir(wasmedge_image_temp):
-        wasmedge_image_temp_dir = join(wasmedge_image_temp, dir)
-        for file in listdir(wasmedge_image_temp_dir):
-            if isdir(join(wasmedge_image_temp_dir, file)) and "wasmedge" == file:
-                copytree(
-                    join(wasmedge_image_temp_dir, file),
-                    join(args.path, "include", "wasmedge"),
-                )
-            elif CONST_lib_ext in file:
-                if isdir(join(args.path, CONST_lib_dir)):
-                    shutil.move(
-                        join(wasmedge_image_temp_dir, file),
-                        join(args.path, CONST_lib_dir, file),
-                    )
-                else:
-                    logging.error(
-                        "%s directory not found", join(args.path, CONST_lib_dir)
-                    )
-                    try:
-                        mkdir(join(args.path, CONST_lib_dir))
-                        shutil.move(
-                            join(wasmedge_image_temp_dir, file),
-                            join(args.path, "lib", file),
-                        )
-                    except:
-                        pass
-            elif isdir(join(wasmedge_image_temp_dir, file)):
-                copytree(
-                    join(wasmedge_image_temp_dir, file),
-                    join(args.path, file),
-                )
-            else:
-                shutil.move(
-                    join(wasmedge_image_temp_dir, file),
-                    join(args.path, "bin", file),
-                )
-
-    fix_gnu_sparse(args)
-
-    return 0
-
-
 def install_tensorflow_extension(
     args,
     compat,
-    download_tf_=False,
-    download_tf_lite_=False,
     download_tf_deps_=False,
     download_tf_lite_deps_=False,
-    download_tf_tools_=False,
 ):
     global CONST_release_pkg, CONST_lib_ext, CONST_lib_dir, CONST_env_path
 
-    download_tf = download_tf_
-    download_tf_lite = download_tf_lite_
     download_tf_deps = download_tf_deps_
     download_tf_lite_deps = download_tf_lite_deps_
-    download_tf_tools = download_tf_tools_
 
     logging.debug(
-        "install_tensorflow_extension: %s %s %s %s %s",
-        download_tf,
-        download_tf_lite,
+        "install_tensorflow_extension: %s %s",
         download_tf_deps,
         download_tf_lite_deps,
-        download_tf_tools,
     )
 
-    if VersionString(args.version).compare("0.13.0") >= 0:
-        # if greater than 0.13.0 then No WasmEdge-tensorflow and WasmEdge-tensorflow-tools
-        download_tf = False
-        download_tf_lite = False
-        download_tf_tools = False
-        logging.debug("No WasmEdge-tensorflow and WasmEdge-tensorflow-tools")
-
-    if (
-        not get_remote_version_availability(
-            "second-state/WasmEdge-tensorflow", args.tf_version
-        )
-        and download_tf
-    ):
-        logging.debug(
-            "Tensorflow extension version not found: {0}".format(args.tf_version)
-        )
-        download_tf = False
-
     if (
         not get_remote_version_availability(
             "second-state/WasmEdge-tensorflow-deps", args.tf_deps_version
@@ -839,37 +707,7 @@ def install_tensorflow_extension(
         )
         download_tf_deps = False
 
-    if (
-        not get_remote_version_availability(
-            "second-state/WasmEdge-tensorflow", args.tf_tools_version
-        )
-        and download_tf_tools
-    ):
-        logging.debug(
-            "Tensorflow Tools version not found: {0}".format(args.tf_tools_version)
-        )
-        download_tf_tools = False
-
-    if compat.prefix() + TENSORFLOW not in SUPPORTED_EXTENSIONS_VERSION:
-        logging.error(
-            "Tensorflow extensions not compatible: {0}".format(compat.prefix())
-        )
-        return -1
-    elif (
-        SUPPORTED_EXTENSIONS_VERSION[compat.prefix() + TENSORFLOW].compare(
-            args.tf_version
-        )
-        > 0
-    ):
-        logging.error(
-            "Min tensorflow extensions version: {0}".format(
-                SUPPORTED_EXTENSIONS_VERSION[compat.prefix() + TENSORFLOW],
-            )
-        )
-        return -1
-
     if compat.machine == "aarch64":
-        download_tf = False
         download_tf_deps = False
         logging.warning(
             "Cannot download WasmEdge Tensorflow, Tools & Deps because it is aarch64"
@@ -883,20 +721,6 @@ def install_tensorflow_extension(
         local_release_package = compat.release_package_wasmedge
         logging.debug("Downloading dist package: {0}".format(local_release_package))
 
-    if download_tf:
-        tf_pkg = "WasmEdge-tensorflow-" + args.tf_version + "-" + local_release_package
-        logging.info("Downloading tensorflow extension")
-        download_url(CONST_urls[TENSORFLOW], join(TEMP_PATH, tf_pkg), show_progress)
-        # Extract archive
-        extract_archive(
-            join(TEMP_PATH, tf_pkg),
-            args.path,
-            join(TEMP_PATH, "WasmEdge-tensorflow"),
-            env_file_path=CONST_env_path,
-            remove_finished=True,
-        )
-        copytree(join(TEMP_PATH, "WasmEdge-tensorflow"), args.path)
-
     if download_tf_deps:
         tf_deps_pkg = (
             "WasmEdge-tensorflow-deps-TF-"
@@ -920,24 +744,6 @@ def install_tensorflow_extension(
         )
         copytree(join(TEMP_PATH, "WasmEdge-tensorflow-deps"), args.path)
 
-    if download_tf_lite:
-        tf_lite_pkg = (
-            "WasmEdge-tensorflowlite-" + args.tf_version + "-" + local_release_package
-        )
-        logging.info("Downloading tensorflow-lite extension")
-        download_url(
-            CONST_urls[TENSORFLOW_LITE], join(TEMP_PATH, tf_lite_pkg), show_progress
-        )
-        # Extract archive
-        extract_archive(
-            join(TEMP_PATH, tf_lite_pkg),
-            args.path,
-            join(TEMP_PATH, "WasmEdge-tensorflow-lite"),
-            env_file_path=CONST_env_path,
-            remove_finished=True,
-        )
-        copytree(join(TEMP_PATH, "WasmEdge-tensorflow-lite"), args.path)
-
     if download_tf_lite_deps:
         tf_deps_lite_pkg = (
             "WasmEdge-tensorflow-deps-TFLite-"
@@ -964,30 +770,6 @@ def install_tensorflow_extension(
 
         copytree(join(TEMP_PATH, "WasmEdge-tensorflow-lite-deps"), args.path)
 
-    if download_tf_tools:
-        tf_tools_pkg = (
-            "WasmEdge-tensorflow-tools-"
-            + args.tf_tools_version
-            + "-"
-            + CONST_release_pkg
-        )
-
-        logging.info("Downloading tensorflow-tools extension")
-        download_url(
-            CONST_urls[TENSORFLOW_TOOLS], join(TEMP_PATH, tf_tools_pkg), show_progress
-        )
-
-        # Extract archive
-        extract_archive(
-            join(TEMP_PATH, tf_tools_pkg),
-            join(args.path, "bin"),
-            join(TEMP_PATH, "WasmEdge-tensorflow-tools", "bin"),
-            env_file_path=CONST_env_path,
-            remove_finished=True,
-        )
-
-        copytree(join(TEMP_PATH, "WasmEdge-tensorflow-tools"), args.path)
-
     fix_gnu_sparse(args)
 
     all_files = run_shell_command("ls -R {0}".format(TEMP_PATH))
@@ -1118,36 +900,6 @@ def install_tensorflow_extension(
                             join(args.path, "bin", _file),
                         )
 
-    if download_tf_tools and download_tf:
-        # Check if wasmedge binary works
-        wasmedge_tf_output = run_shell_command(
-            ". {0}/env &&{0}/bin/wasmedge-tensorflow --version".format(args.path)
-        )
-
-        if args.tf_version in wasmedge_tf_output:
-            logging.info("WasmEdge Successfully installed")
-        else:
-            logging.critical(
-                "WasmEdge Tensorflow installation incorrect: {0}".format(
-                    wasmedge_tf_output
-                )
-            )
-
-    if download_tf_tools and download_tf_lite:
-        # Check if wasmedge binary works
-        wasmedge_tf_lite_output = run_shell_command(
-            ". {0}/env && {0}/bin/wasmedge-tensorflow-lite --version".format(args.path)
-        )
-
-        if args.tf_version in wasmedge_tf_lite_output:
-            logging.info("WasmEdge Tensorflow Lite Successfully installed")
-        else:
-            logging.critical(
-                "WasmEdge Tensorflow installation incorrect: {0}".format(
-                    wasmedge_tf_lite_output
-                )
-            )
-
     return 0
 
 
@@ -1168,6 +920,28 @@ def install_plugins(args, compat):
             if plugin_name.find(":") != -1:
                 plugin_name, plugin_version_supplied = plugin_name.split(":")
 
+            # Deprecated rustls after 0.14.0
+            # Only allow users to install rustls with 0.13.5
+            if (
+                plugin_name.startswith(WASMEDGE_RUSTLS)
+                and compat.version.compare("0.13.5") != 0
+            ):
+                logging.warning("WasmEdge Rustls plugin is only available in 0.13.5")
+                logging.warning("Please use -v 0.13.5 as a workaround")
+                logging.warning("Skip installing WasmEdge Rustls plugin")
+                continue
+
+            # Deprecated wasi-logging after 0.14.1-rc.1
+            if (
+                plugin_name.startswith(WASI_LOGGING)
+                and compat.version.compare("0.14.1-rc.1") != -1
+            ):
+                logging.warning(
+                    "WASI-Logging plugin is bundled into libwasmedge in 0.14.1-rc.1"
+                )
+                logging.warning("No need to install the WASI-Logging plugin")
+                continue
+
             # Split the WASI-NN-GGML plugin and the others
             if plugin_name.startswith(WASI_NN_GGML):
                 # Re-write the plugin name if the build number is supplied
@@ -1312,14 +1086,6 @@ def set_consts(args, compat):
         local_release_package_tf = compat.release_package_wasmedge
         logging.debug("Tensorflow release pkg: {0}".format(local_release_package_tf))
 
-    local_release_package_im = CONST_release_pkg
-
-    # From WasmEdge 0.11.1, we have the Ubuntu release.
-    # Installation of ubuntu version extensions when the ubuntu version of WasmEdge selected.
-    if VersionString(args.image_version).compare("0.11.1") >= 0:
-        local_release_package_im = compat.release_package_wasmedge
-        logging.debug("Image release pkg: {0}".format(local_release_package_im))
-
     CONST_urls = {
         WASMEDGE: "https://github.com/WasmEdge/WasmEdge/releases/download/{0}/WasmEdge-{0}-{1}".format(
             args.version, compat.release_package_wasmedge
@@ -1327,24 +1093,12 @@ def set_consts(args, compat):
         WASMEDGE_UNINSTALLER: "https://raw.githubusercontent.com/WasmEdge/WasmEdge/{0}/utils/uninstall.sh".format(
             args.uninstall_script_tag
         ),
-        IMAGE: "https://github.com/second-state/WasmEdge-image/releases/download/{0}/WasmEdge-image-{0}-{1}".format(
-            args.image_version, local_release_package_im
-        ),
         TENSORFLOW_DEPS: "https://github.com/second-state/WasmEdge-tensorflow-deps/releases/download/{0}/WasmEdge-tensorflow-deps-TF-{0}-{1}".format(
             args.tf_deps_version, CONST_release_pkg
         ),
         TENSORFLOW_LITE_DEPS: "https://github.com/second-state/WasmEdge-tensorflow-deps/releases/download/{0}/WasmEdge-tensorflow-deps-TFLite-{0}-{1}".format(
             args.tf_deps_version, CONST_release_pkg
         ),
-        TENSORFLOW: "https://github.com/second-state/WasmEdge-tensorflow/releases/download/{0}/WasmEdge-tensorflow-{0}-{1}".format(
-            args.tf_version, local_release_package_tf
-        ),
-        TENSORFLOW_LITE: "https://github.com/second-state/WasmEdge-tensorflow/releases/download/{0}/WasmEdge-tensorflowlite-{0}-{1}".format(
-            args.tf_version, local_release_package_tf
-        ),
-        TENSORFLOW_TOOLS: "https://github.com/second-state/WasmEdge-tensorflow-tools/releases/download/{0}/WasmEdge-tensorflow-tools-{0}-{1}".format(
-            args.tf_tools_version, CONST_release_pkg
-        ),
     }
 
 
@@ -1355,6 +1109,8 @@ def run_shell_command(cmd):
     except subprocess.CalledProcessError as e:
         if "Cannot detect installation path" in str(e.output):
             logging.warning("Uninstaller did not find previous installation")
+        elif "ldconfig" in str(e.cmd):
+            logging.debug("Cannot detect libcudart via ldconfig")
         elif "nvcc" in str(e.cmd):
             logging.debug("Cannot detect CUDA via nvcc")
         else:
@@ -1402,19 +1158,19 @@ class Compat:
         machine=platform.machine(),
         dist_=None,
         version=None,
-        extensions=None,
     ):
         self.platform = platform_  # Linux, Darwin
         self.machine = machine  # x86_64, arm
         self.version = VersionString(version)
-        self.extensions = extensions
         self.release_package = None
         self.install_package_name = None
         self.lib_extension = None
         self.ld_library_path = None
         self.dist = dist_
         self.release_package_wasmedge = None
-        self.cuda = check_nvcc(self.platform) or check_nvidia_smi(self.platform)
+        self.cuda = check_libcudart(self.platform) and (
+            check_nvcc(self.platform) or check_nvidia_smi(self.platform)
+        )
 
         if self.platform == "Linux":
             self.install_package_name = "WasmEdge-{0}-Linux".format(self.version)
@@ -1492,10 +1248,8 @@ class Compat:
                 self.dist = "darwin"
 
     def __str__(self):
-        return (
-            "Platform:{0}\nMachine:{1}\nVersion:{2}\nExtensions:{3}\nDist:{4}\n".format(
-                self.platform, self.machine, self.version, self.extensions, self.dist
-            )
+        return "Platform:{0}\nMachine:{1}\nVersion:{2}\nDist:{3}\n".format(
+            self.platform, self.machine, self.version, self.dist
         )
 
     if sys.version_info[0] == 2:
@@ -1513,31 +1267,7 @@ class Compat:
             reraise(Exception("Unsupported platform: {0}".format(self.platform)))
         if self.machine not in SUPPORTED_PLATFORM_MACHINE[self.platform]:
             reraise(Exception("Unsupported machine: {0}".format(self.machine)))
-        if self.extensions is not None and len(self.extensions) > 0:
-            if not (
-                set(self.extensions)
-                <= set(SUPPORTED_EXTENSIONS[self.platform + self.machine])
-            ):
-                logging.error("Supported platforms and corresponding extensions:")
-                for key in SUPPORTED_EXTENSIONS:
-                    _extensions = None
-                    if len(SUPPORTED_EXTENSIONS[key]) >= 1:
-                        _extensions = ",".join(SUPPORTED_EXTENSIONS[key])
-                    else:
-                        _extensions = "None"
-                    logging.error(
-                        "Platform: {0} Supported Extensions: {1}".format(
-                            key, _extensions
-                        )
-                    )
-                reraise(
-                    Exception(
-                        "Extensions not supported: {0}. Supported extensions: {1}".format(
-                            self.extensions,
-                            SUPPORTED_EXTENSIONS[self.platform + self.machine],
-                        )
-                    )
-                )
+
         if (
             self.version.compare(
                 version2=SUPPORTED_MIN_VERSION[self.platform + self.machine].version
@@ -1574,7 +1304,6 @@ def main(args):
 
     compat = Compat(
         version=args.version,
-        extensions=args.extensions,
         platform_=args.platform,
         machine=args.machine,
         dist_=args.dist,
@@ -1593,18 +1322,13 @@ def main(args):
             args.plugins = PLUGINS_AVAILABLE[:]
             logging.debug("Selected all of the available plugins: %s", args.plugins)
 
-    if len(args.extensions) >= 1 and compat.version.compare("0.13.0") != -1:
-        logging.warning(
-            "Extensions exist only for versions below 0.13.0, use plugins instead"
-        )
-
     if compat:
         logging.info("Compatible with current configuration")
 
         set_consts(args, compat)
 
-        if compat.version.compare("0.10.0") == -1:
-            logging.error("Please install the 0.10.0 or above versions.")
+        if compat.version.compare("0.13.0") == -1:
+            logging.error("Please install the 0.13.0 or above versions.")
             exit(1)
 
         # Run uninstaller
@@ -1690,37 +1414,6 @@ def main(args):
                 "WasmEdge installation incorrect: {0}".format(wasmedge_output)
             )
 
-        if (
-            IMAGE in args.extensions
-            or "all" in args.extensions
-            and VersionString(args.version).compare("0.13.0") == -1
-        ):
-            if install_image_extension(args, compat) != 0:
-                logging.error("Error in installing image extensions")
-            else:
-                logging.info("Image extension installed")
-
-        if (
-            TENSORFLOW in args.extensions
-            or "all" in args.extensions
-            and VersionString(args.version).compare("0.13.0") == -1
-        ):
-            if (
-                install_tensorflow_extension(
-                    args,
-                    compat,
-                    download_tf_=True,
-                    download_tf_deps_=True,
-                    download_tf_lite_=True,
-                    download_tf_lite_deps_=True,
-                    download_tf_tools_=True,
-                )
-                != 0
-            ):
-                logging.error("Error in installing tensorflow extensions")
-            else:
-                logging.info("Tensorflow extension installed")
-
         install_plugins(args, compat)
 
         ldconfig(args, compat)
@@ -1746,17 +1439,7 @@ def main(args):
 
 if __name__ == "__main__":
     parser = argparse.ArgumentParser(
-        description="WasmEdge installation, uninstallation and extensions install"
-    )
-    parser.add_argument(
-        "-e",
-        "--extension",
-        dest="extensions",
-        choices=EXTENSIONS.append("all"),
-        required=False,
-        default=[],
-        nargs="*",
-        help="Supported Extensions - {0}".format(EXTENSIONS),
+        description="WasmEdge installation, uninstallation and plugins install"
     )
     parser.add_argument(
         "-v",
@@ -1827,20 +1510,6 @@ if __name__ == "__main__":
         default=None,
         help="Tensorflow and tensorflow lite deps version",
     )
-    parser.add_argument(
-        "--tf-tools-version",
-        dest="tf_tools_version",
-        required=False,
-        default=None,
-        help="Tensorflow and tensorflow lite tools version",
-    )
-    parser.add_argument(
-        "--image-version",
-        dest="image_version",
-        required=False,
-        default=None,
-        help="Image extension version",
-    )
     parser.add_argument(
         "--platform",
         "--os",
@@ -1889,11 +1558,5 @@ if __name__ == "__main__":
         else:
             reraise("Should not reach here")
 
-    if args.tf_tools_version is None:
-        args.tf_tools_version = args.version
-
-    if args.image_version is None:
-        args.image_version = args.version
-
     logging.debug("Python Version: %s", sys.version_info)
     main(args)
